package ru.dbotthepony.mc.otm.menu.input

import net.minecraft.world.entity.player.Player
import ru.dbotthepony.mc.otm.client.minecraft
import ru.dbotthepony.mc.otm.core.GetterSetter
import ru.dbotthepony.mc.otm.menu.MatteryMenu
import kotlin.reflect.KMutableProperty0

/**
 * Getting and setting values should ONLY be done clientside
 */
abstract class AbstractPlayerInputWithFeedback<V> : GetterSetter<V> {
	abstract val input: MatteryMenu.PlayerInput<V>
	abstract val value: V

	override fun get(): V {
		return value
	}

	override fun accept(t: V) {
		input.checkedInput(t, minecraft.player as Player?)
	}

	var supplier: (() -> V)? = null
	var consumer: ((V) -> Unit)? = null

	fun withSupplier(func: () -> V): AbstractPlayerInputWithFeedback<V> {
		supplier = func
		return this
	}

	fun withConsumer(func: (V) -> Unit): AbstractPlayerInputWithFeedback<V> {
		consumer = func
		return this
	}

	fun with(state: KMutableProperty0<V>): AbstractPlayerInputWithFeedback<V> {
		withConsumer { state.set(it) }
		withSupplier { state.get() }
		return this
	}

	fun with(state: GetterSetter<V>): AbstractPlayerInputWithFeedback<V> {
		withConsumer(state::accept)
		withSupplier(state::get)
		return this
	}

	fun clear(): AbstractPlayerInputWithFeedback<V> {
		supplier = null
		consumer = null
		return this
	}

	/**
	 * shortcut to checked input of [input]
	 */
	fun input(newValue: V) {
		input.checkedInput(newValue, minecraft.player as Player?)
	}

	/**
	 * checks if local (client) player can modify this input
	 */
	fun check(): Boolean {
		return input.allowSpectators || (minecraft.player as Player?)?.isSpectator == false
	}
}
